package FCC::App::Actions::Moddtljsn;
$VERSION = 1.00;
use strict;
use warnings;
use utf8;
use base qw(FCC::App::Actions::_Super);
use JSON;

sub dispatch {
    my ($self) = @_;

    # パラメーターを取得
    my $in = $self->get_input_data( [ 'name', 'path' ] );
    unless ( $in->{name} && $in->{path} ) {
        $self->print_json( { error => 'パラメーターが不正です。' }, 400 );
        return;
    }

    my $path = $in->{path};
    my $name = $in->{name};

    # パラメーター `name` の値をチェック
    if ( !$name || $name =~ /[^a-zA-Z0-9\_\:]/ ) {
        $self->print_json( { error => "パラメーター `nae` の値が不正です: ${path}" }, 400 );
        return;
    }

    # パラメーター `path` の値をチェック
    if (  !$path
        || $path !~ /\.pm$/
        || $path =~ /\.\./
        || $path =~ /[^a-zA-Z0-9\/\.\_\-]/ )
    {
        $self->print_json( { error => "パラメーター `path` の値が不正です: ${path}" }, 400 );
        return;
    }

    # パラメーター `path`` の値が @INC のパスの中かどうかをチェック
    my @path_parts = split( /\//, $path );
    pop(@path_parts);
    my $dpath         = join( '/', @path_parts );
    my $libpath_list  = $self->get_libpath_list();
    my $is_valid_path = 0;
    for my $path ( @{$libpath_list} ) {
        if ( $dpath =~ /^\Q${path}\E/ ) {
            $is_valid_path = 1;
            last;
        }
    }
    unless ($is_valid_path) {
        $self->print_json( { error => "パラメーター `path` の値が不正です: ${path}" }, 400 );
        return;
    }

    # モジュールのバージョンを取得
    my $version = '';
    eval {
        $version = `$^X -m${name} -e 'print \$${name}::VERSION' 2>/dev/null`;
    };
    if($@) {
        $self->print_json( { error => "指定の名前のモジュールが見つかりませんでした: ${name}" }, 404 );
        return;
    }

    my $data = {
        name    => $in->{name},
        path    => $in->{path},
        version => $version,
    };

    $self->print_json($data);
}

# Perl モジュール検索パス
sub get_libpath_list {
    my ($self) = @_;
    my $this_path = $self->{conf}->{CGI_DIR};
    my @list;
    for my $path (@INC) {
        if ( $path eq '.' ) {
            next;
        }
        unless ( $path =~ /^\Q${this_path}\E/ ) {
            push( @list, $path );
        }
    }
    return \@list;
}

1;
