package FCC::App::Actions::_Super;
$VERSION = 1.00;
use strict;
use warnings;
use utf8;
use base qw(FCC::App::_Super);
use Encode;
use Template;
use HTTP::Status;

sub redirect {
    my ( $self, $url, $headers ) = @_;
    if ($headers) {
        while ( my ( $k, $v ) = each %{$headers} ) {
            print "${k}: ${v}\n";
        }
    }
    print "Location: ${url}\n\n";
    exit;
}

sub output_template {
    my ( $self, $vars ) = @_;
    my $data = {};
    if ( $vars && ref($vars) eq 'HASH' ) {
        while ( my ( $k, $v ) = each %{$vars} ) {
            $data->{$k} = $v;
        }
    }

    $data->{CGI_URL}  = $self->{conf}->{CGI_URL};    # https://xxx/app/index.cgi
    $data->{CGI_PATH} = $self->{conf}->{CGI_PATH};   # /app/index.cgi
    $data->{CGI_APP_PATH}    = $self->{conf}->{CGI_APP_PATH};      # /app
    $data->{CGI_SSL}         = $self->{conf}->{CGI_SSL};
    $data->{static_url}      = $self->{conf}->{static_url};
    $data->{product_name}    = $self->{conf}->{product_name};
    $data->{product_version} = $self->{conf}->{product_version};
    $data->{product_url}     = $self->{conf}->{product_url};
    $data->{author_name}     = $self->{conf}->{author_name};
    $data->{author_url}      = $self->{conf}->{author_url};
    $data->{copyright}       = $self->{conf}->{copyright};
    $data->{copyright_url}   = $self->{conf}->{copyright_url};
    $data->{twitter_url}     = $self->{conf}->{twitter_url};

    if ( $self->{session} && $self->{session}->id() ) {
        my $user = $self->{session}->param('user');
        $data->{user} = $user || {};
        if ($user) {
            if ( $user->{auth_time} ) {
                my $t = localtime( $user->{auth_time} );
                $user->{auth_time_ymd} = $t->ymd;
                $user->{auth_time_hms} = $t->hms;
            }
        }
    }

    my $file = $self->{conf}->{MODE} . '.html';
    unless ( -e $self->{conf}->{TEMPLATE_DIR} . '/' . $file ) {
        my $msg = '指定のページは見つかりませんでした。';
        $self->print_text( $msg, 404 );
    }

    my $output = '';
    my $tt     = $self->{template};
    $tt->process( $file, $data, \$output ) || die $tt->error();
    return $output;
}

sub print_text {
    my ( $self, $msg, $status_code ) = @_;
    print $self->{cgi}->header(
        -status  => $status_code,
        -type    => 'text/plain',
        -charset => 'utf-8',
    );
    print $msg;
    exit;
}

sub print_html {
    my ( $self, $body, $status_code, $hdrs_ref ) = @_;
    my %hdrs;
    if ($hdrs_ref) {
        while ( my ( $k, $v ) = each %{$hdrs_ref} ) {
            $k = lc $k;
            $hdrs{$k} = $v;
        }
    }

    # ヘッダー初期値
    unless ( $hdrs{'content-type'} ) {
        $hdrs{'content-type'} = 'text/html; charset=utf-8';
    }

    # ボディを出力用にエンコード
    $body = Encode::encode( 'utf8', $body );

    # ヘッダーに Content-Length をセット
    $hdrs{'content-length'} = length $body;

    # 出力
    unless ($status_code) {
        $status_code = 200;
    }
    my $status_text = HTTP::Status::status_message($status_code);
    print "Status: ${status_code} ${status_text}\n";
    while ( my ( $k, $v ) = each %hdrs ) {
        my $name = $k;
        $name =~ s/^([a-z]+)\-([a-z]+)$/\u$1\-\u$2/;
        if ( ref($v) && ref($v) eq "ARRAY" ) {
            for my $str ( @{$v} ) {
                print "${name}: ${str}\n";
            }
        }
        else {
            print "${name}: ${v}\n";
        }
    }
    print "\n";
    print $body;
    exit;
}

sub print_json {
    my ( $self, $data, $status_code, $hdrs_ref ) = @_;
    my %hdrs;
    if ($hdrs_ref) {
        while ( my ( $k, $v ) = each %{$hdrs_ref} ) {
            $k = lc $k;
            $hdrs{$k} = $v;
        }
    }

    # ヘッダー初期値
    unless ( $hdrs{'content-type'} ) {
        $hdrs{'content-type'} = 'application/json; charset=utf-8';
    }

    # ボディを出力用にエンコード
    my $body = JSON::encode_json($data);

    # ヘッダーに Content-Length をセット
    $hdrs{'content-length'} = length $body;

    # 出力
    unless ($status_code) {
        $status_code = 200;
    }
    my $status_text = HTTP::Status::status_message($status_code);
    print "Status: ${status_code} ${status_text}\n";
    while ( my ( $k, $v ) = each %hdrs ) {
        my $name = $k;
        $name =~ s/^([a-z]+)\-([a-z]+)$/\u$1\-\u$2/;
        if ( ref($v) && ref($v) eq "ARRAY" ) {
            for my $str ( @{$v} ) {
                print "${name}: ${str}\n";
            }
        }
        else {
            print "${name}: ${v}\n";
        }
    }
    print "\n";
    print $body;
    exit;
}

sub get_input_data {
    my ( $self, $target_cols, $array_cols ) = @_;
    my @cols;
    if ( $target_cols && ref($target_cols) eq "ARRAY" ) {
        @cols = @{$target_cols};
    }
    else {
        @cols = $self->{cgi}->param();
    }
    my %arys;
    if ( $array_cols && ref($array_cols) eq "ARRAY" ) {
        for my $col ( @{$array_cols} ) {
            $arys{$col} = 1;
        }
    }
    my $in = {};
    for my $col (@cols) {
        my $v;
        if ( $arys{$col} ) {
            my @raw_list = $self->{cgi}->param($col);
            my @list;
            for my $raw (@raw_list) {
                push( @list, Encode::decode( 'utf8', $raw ) );
            }
            $v = \@list;
        }
        else {
            $v = $self->{cgi}->param($col);
            if ( defined $v ) {
                $v = Encode::decode( 'utf8', $v );
                $self->conv_line_breaks( \$v );
            }
            else {
                $v = "";
            }
        }
        $in->{$col} = $v;
    }
    return $in;
}

sub conv_line_breaks {
    my ( $self, $str_ref ) = @_;
    ${$str_ref} =~ s/\r\n/\n/g;
    ${$str_ref} =~ s/\r/\n/g;
}

1;
